#include "FormatHandler.h"
#include "Data.h"
#include "FakeXmlParser.h"

#include "DataInt8.h"
#include "DataUInt16.h"
#include "DataFloat32.h"

using namespace std;

FormatHandler::FormatHandler()
{
	// We create the data vector according to XML config
	vector<XmlDataElement> config = parse_xml_data();

	vector<XmlDataElement>::const_iterator it;
	for (it = config.begin(); it != config.end(); ++it) {
		Data *d;
		if (it->type == "int8") 
			 d = new DataInt8(it->name, it->len);
		else if (it->type == "uint16") 
			 d = new DataUInt16(it->name, it->len);
		else if (it->type == "float32") 
			 d = new DataFloat32(it->name, it->len);
		else {
			cout << "Unknown data type = '" << it->type << 
			        "' ! Skipping" << endl;
			continue;
		}
		data_vector.push_back(d);
	}
}

FormatHandler::~FormatHandler()
{
	// We have to destroy the data vector properly
	vector<Data*>::iterator it;
	for (it = data_vector.begin(); it != data_vector.end(); ++it)
		delete *it;
}

void FormatHandler::format(uint8_t* rawdata, unsigned long len)
{
	vector<Data*>::iterator it;
	for (it = data_vector.begin(); it != data_vector.end(); ++it) {
		unsigned long used_bytes = (*it)->format_data(rawdata, len);
		len -= used_bytes;
		rawdata += used_bytes;
	}
}
