#ifndef __DataFactory_h__
#define __DataFactory_h__

#include <string>
#include <map>

class Data;

class DataFactory {
public:
	typedef Data* (*creator_type_t)(const std::string&, unsigned long);

	DataFactory(const std::string& id, creator_type_t func);
	static Data* create(const std::string& id, const std::string& name,
	                    unsigned long len);
private:
	static std::map<std::string, creator_type_t>* lookup_table;
};

#define REGISTER_DATA(class_name, type_id)                                     \
static Data* reg_func_##class_name(const std::string& n, unsigned long l)      \
{                                                                              \
	return new class_name(n, l);					       \
}                                                                              \
static DataFactory reg_obj_##class_name(type_id, reg_func_##class_name);


#endif // __DataFactory_h__
