#include "FactoryExample5.h"
#include <iostream>

using namespace std;

map<std::string, DisneyFactory::creator_type_t> DisneyFactory::lookup_table;

DisneyFactory::DisneyFactory(const string& id, creator_type_t func)
{
	cout << "Factory: registrating creator for id=" << id << endl;
	if (lookup_table.find(id) != lookup_table.end()) {
		cout << "Error: ID already in look-up table" << endl;
		return;
	}
	lookup_table[id] = func;
}

DisneyProduct* DisneyFactory::create(const string& id)
{
	if (lookup_table.find(id) == lookup_table.end())
		return NULL;
	return lookup_table[id]();
}

// ORDER FIASCO!
DisneyFactory register_goofy("goofy", Goofy::create);
DisneyFactory register_pluto("pluto", Pluto::create);

DisneyUser::DisneyUser()
{
	const char* config[] = {"goofy", "pluto", "pluto", "invalid_id"};

	for (size_t i = 0; i < 4; i++) {
		DisneyProduct *p = DisneyFactory::create(config[i]);
		if (p)
			toons.push_back(p);
	}
}

DisneyUser::~DisneyUser() 
{
	vector<DisneyProduct*>::iterator it;
	for (it = toons.begin(); it != toons.end(); ++it)
		delete *it;
}

void DisneyUser::animate(void)
{
	vector<DisneyProduct*>::const_iterator it;
	for (it = toons.begin(); it != toons.end(); ++it)
		cout << "Hullo, my name is " << (*it)->get_name() << endl;
}

int main(void)
{
	cout << "Example: " << __FILE__ << endl << endl;

	{
	DisneyUser u;
	u.animate();
	} // ~DisneyUser() is called here!

	cout << endl << "Example terminates " << endl;

	return 0;
}
